@@ -1,7 +1,7 @@
 /*
  * This file is part of uudeview, the simple and friendly multi-part multi-
- * file uudecoder  program  (c)  1994 by Frank Pilhofer. The author may be
- * contacted by his email address,          fp@informatik.uni-frankfurt.de
+ * file uudecoder  program  (c) 1994-2001 by Frank Pilhofer. The author may
+ * be contacted at fp@fpx.de
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 
 /*
  * This file implements the externally visible functions, as declared
- * in uulib.h, and some internal interfacing functions
+ * in uudeview.h, and some internal interfacing functions
  */
 
 #ifdef HAVE_CONFIG_H
@@ -76,12 +76,12 @@
 #include <io.h>
 #endif
 
-#include <uulib.h>
+#include <uudeview.h>
 #include <uuint.h>
 #include <fptools.h>
 #include <uustring.h>
 
-char * uulib_id = "$Id: uulib.c,v 1.16 1996/11/03 12:49:07 fp Exp $";
+char * uulib_id = "$Id: uulib.c,v 1.8 2002/04/05 21:56:57 root Exp $";
 
 #ifdef SYSTEM_WINDLL
 BOOL _export WINAPI 
@@ -114,15 +114,17 @@ DllEntryPoint (HINSTANCE hInstance, DWORD seginfo,
  * Callback functions and their opaque arguments
  */
 
-void   (*uu_MsgCallback)  (void *, char *, int)         = NULL;
-int    (*uu_BusyCallback) (void *, uuprogress *)        = NULL;
-int    (*uu_FileCallback) (void *, char *, char *, int) = NULL;
-char * (*uu_FNameFilter)  (void *, char *)              = NULL;
-
+void   (*uu_MsgCallback)     (void *, char *, int)         = NULL;
+int    (*uu_BusyCallback)    (void *, uuprogress *)        = NULL;
+int    (*uu_FileCallback)    (void *, char *, char *, int) = NULL;
+char * (*uu_FNameFilter)     (void *, char *)              = NULL;
+char * (*uu_FileNameCallback)(void *, char *, char *);
+;
 void *uu_MsgCBArg  = NULL;
 void *uu_BusyCBArg = NULL;
 void *uu_FileCBArg = NULL;
 void *uu_FFCBArg   = NULL;
+void *uu_FNCBArg;
 
 /*
  * Global variables
@@ -141,6 +143,8 @@ int uu_ignmode = 0;		/* ignore the original file mode            */
 int uu_handletext = 0;		/* do we want text/plain messages           */
 int uu_usepreamble = 0;		/* do we want Mime preambles/epilogues      */
 int uu_tinyb64 = 0;		/* detect short B64 outside of MIME         */
+int uu_remove_input = 0;        /* remove input files after decoding        */
+int uu_more_mime = 0;           /* strictly adhere to MIME headers          */
 
 headercount hlcount = {
   3,			        /* restarting after a MIME body             */
@@ -224,29 +228,29 @@ typedef struct {
 } allomap;
 
 static allomap toallocate[] = {
-  { &uugen_fnbuffer,    1024 },  /* generic filename buffer */
+  { &uugen_fnbuffer,    4096 },  /* generic filename buffer */
   { &uugen_inbuffer,    1024 },  /* generic input data buffer */
   { &uucheck_lastname,   256 },	 /* from uucheck.c */
   { &uucheck_tempname,   256 },
   { &uuestr_itemp,       256 },  /* from uuencode.c:UUEncodeStream() */
-  { &uuestr_otemp,       256 },
+  { &uuestr_otemp,      1024 },
   { &uulib_msgstring,   1024 },  /* from uulib.c:UUMessage() */
-  { &uuncdl_fulline,   256+4 },  /* from uunconc.c:UUDecodeLine(). +4 is for leftover parts */
-  { &uuncdp_oline,     4*257 },  /* from uunconc.c:UUDecodePart() */
+  { &uuncdl_fulline,    1200 },  /* from uunconc.c:UUDecodeLine() */
+  { &uuncdp_oline,      1200 },  /* from uunconc.c:UUDecodePart() */
   { &uunconc_UUxlat,     256 * sizeof (int) },  /* from uunconc.c:toplevel */
   { &uunconc_UUxlen,      64 * sizeof (int) },
   { &uunconc_B64xlat,    256 * sizeof (int) },
   { &uunconc_XXxlat,     256 * sizeof (int) },
   { &uunconc_BHxlat,     256 * sizeof (int) },
-  { &uunconc_save,     3*256 },  /* from uunconc.c:decoding buffer */
+  { &uunconc_save,     3*300 },  /* from uunconc.c:decoding buffer */
   { &uuscan_shlline,    1024 },  /* from uuscan.c:ScanHeaderLine() */
-  { &uuscan_pvvalue,     256 },  /* from uuscan.c:ParseValue() */
-  { &uuscan_phtext,      256 },  /* from uuscan.c:ParseHeader() */
-  { &uuscan_sdline,      256 },  /* from uuscan.c:ScanData() */
-  { &uuscan_sdbhds1,     256 },
-  { &uuscan_sdbhds2,     256 },
-  { &uuscan_spline,      256 },  /* from uuscan.c:ScanPart() */
-  { &uuutil_bhwtmp,      256 },  /* from uuutil.c:UUbhwrite() */
+  { &uuscan_pvvalue,     300 },  /* from uuscan.c:ParseValue() */
+  { &uuscan_phtext,      300 },  /* from uuscan.c:ParseHeader() */
+  { &uuscan_sdline,      300 },  /* from uuscan.c:ScanData() */
+  { &uuscan_sdbhds1,     300 },
+  { &uuscan_sdbhds2,     300 },
+  { &uuscan_spline,      300 },  /* from uuscan.c:ScanPart() */
+  { &uuutil_bhwtmp,      300 },  /* from uuutil.c:UUbhwrite() */
   { NULL, 0 }
 };
 
@@ -390,7 +394,7 @@ UUInitialize (void)
        * areas (uulib_msgstring) in UUMessage()
        */
       for (aiter=toallocate; aiter->ptr; aiter++) {
-	FP_free (*(aiter->ptr));
+	_FP_free (*(aiter->ptr));
       }
       return UURET_NOMEM;
     }
@@ -416,7 +420,7 @@ UUGetOption (int option, int *ivalue, char *cvalue, int clength)
 
   switch (option) {
   case UUOPT_VERSION:
-    FP_strncpy (cvalue, uulibversion, clength);
+    _FP_strncpy (cvalue, uulibversion, clength);
     result = 0;
     break;
   case UUOPT_FAST:
@@ -456,7 +460,7 @@ UUGetOption (int option, int *ivalue, char *cvalue, int clength)
     result = uu_overwrite;
     break;
   case UUOPT_SAVEPATH:
-    FP_strncpy (cvalue, uusavepath, clength);
+    _FP_strncpy (cvalue, uusavepath, clength);
     result = 0;
     break;
   case UUOPT_PROGRESS:
@@ -484,9 +488,17 @@ UUGetOption (int option, int *ivalue, char *cvalue, int clength)
     result = uu_tinyb64;
     break;
   case UUOPT_ENCEXT:
-    FP_strncpy (cvalue, uuencodeext, clength);
+    _FP_strncpy (cvalue, uuencodeext, clength);
     result = 0;
     break;
+  case UUOPT_REMOVE:
+    if (ivalue) *ivalue = uu_remove_input;
+    result = uu_remove_input;
+    break;
+  case UUOPT_MOREMIME:
+    if (ivalue) *ivalue = uu_more_mime;
+    result = uu_more_mime;
+    break;
   default:
     return -1;
   }
@@ -522,8 +534,8 @@ UUSetOption (int option, int ivalue, char *cvalue)
     uu_overwrite      = ivalue;
     break;
   case UUOPT_SAVEPATH:
-    FP_free (uusavepath);
-    uusavepath = FP_strdup (cvalue);
+    _FP_free (uusavepath);
+    uusavepath = _FP_strdup (cvalue);
     break;
   case UUOPT_IGNMODE:
     uu_ignmode = ivalue;
@@ -538,8 +550,14 @@ UUSetOption (int option, int ivalue, char *cvalue)
     uu_tinyb64 = ivalue;
     break;
   case UUOPT_ENCEXT:
-    FP_free (uuencodeext);
-    uuencodeext = FP_strdup (cvalue);
+    _FP_free (uuencodeext);
+    uuencodeext = _FP_strdup (cvalue);
+    break;
+  case UUOPT_REMOVE:
+    uu_remove_input = ivalue;
+    break;
+  case UUOPT_MOREMIME:
+    uu_more_mime = ivalue;
     break;
   default:
     return UURET_ILLVAL;
@@ -599,6 +617,16 @@ UUSetFNameFilter (void *opaque,
   return UURET_OK;
 }
 
+int UUEXPORT
+UUSetFileNameCallback (void *opaque,
+		       char * (*func) (void *, char *, char *))
+{
+  uu_FileNameCallback = func;
+  uu_FNCBArg          = opaque;
+
+  return UURET_OK;
+}
+
 /*
  * Return a pointer to the nth element of the GlobalFileList
  * zero-based, returns NULL if item is too large.
@@ -676,10 +704,10 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
       UUMessage (uulib_id, __LINE__, UUMSG_WARNING,
 		 uustring (S_OUT_OF_MEMORY), sizeof (itbd));
     }
-    else if ((killem->fname = FP_strdup (filename)) == NULL) {
+    else if ((killem->fname = _FP_strdup (filename)) == NULL) {
       UUMessage (uulib_id, __LINE__, UUMSG_WARNING,
 		 uustring (S_OUT_OF_MEMORY), strlen(filename)+1);
-      FP_free (killem);
+      _FP_free (killem);
     }
     else {
       killem->NEXT = ftodel;
@@ -693,7 +721,7 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
   progress.fsize    = (long) ((finfo.st_size>0)?finfo.st_size:-1);
   progress.percent  = 0;
   progress.foffset  = 0;
-  FP_strncpy (progress.curfile,
+  _FP_strncpy (progress.curfile,
 	       (strlen(filename)>255)?
 	       (filename+strlen(filename)-255):filename,
 	       256);
@@ -739,6 +767,7 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
     }
 
     if ((loaded->uudet == QP_ENCODED || loaded->uudet == PT_ENCODED) &&
+	(loaded->filename == NULL || *(loaded->filename) == '\0') &&
 	!uu_handletext && (loaded->flags&FL_PARTIAL)==0) {
       /*
        * Don't want text
@@ -897,7 +926,7 @@ UUDecodeFile (uulist *thefile, char *destname)
   if (destname)
     strcpy (uugen_fnbuffer, destname);
   else {
-    sprintf (uugen_fnbuffer, "%s%s",
+    sprintf (uugen_fnbuffer, "%.1024s%.3071s",
 	     (uusavepath)?uusavepath:"",
 	     UUFNameFilter ((thefile->filename)?
 			    thefile->filename:"unknown.xxx"));
@@ -924,8 +953,22 @@ UUDecodeFile (uulist *thefile, char *destname)
     return UURET_IOERR;
   }
 
+  /* try rename() shortcut first */
+  if (!rename (thefile->binfile, uugen_fnbuffer))
+    {
+      mode_t mask = 0000; /* there is a slight window here anyway */
+#if HAVE_UMASK
+      mask = umask (0022); umask (mask);
+#endif
+      fclose (source);
+#if HAVE_CHMOD
+      chmod (uugen_fnbuffer, thefile->mode & ~mask);
+#endif
+      goto skip_copy;
+    }
+
   progress.action   = 0;
-  FP_strncpy (progress.curfile,
+  _FP_strncpy (progress.curfile,
 	       (strlen(uugen_fnbuffer)>255)?
 	       (uugen_fnbuffer+strlen(uugen_fnbuffer)-255):uugen_fnbuffer,
 	       256);
@@ -1005,7 +1048,9 @@ UUDecodeFile (uulist *thefile, char *destname)
 	       thefile->binfile,
 	       strerror (uu_errno = errno));
   }
-  FP_free (thefile->binfile);
+
+skip_copy:
+  _FP_free (thefile->binfile);
   thefile->binfile = NULL;
   thefile->state  &= ~UUFILE_TMPFILE;
   thefile->state  |=  UUFILE_DECODED;
@@ -1054,7 +1099,7 @@ UUInfoFile (uulist *thefile, void *opaque,
 		 strerror (uu_errno=errno));
       return UURET_IOERR;
     }
-    FP_strncpy (uugen_fnbuffer, thefile->thisfile->data->sfname, 1024);
+    _FP_strncpy (uugen_fnbuffer, thefile->thisfile->data->sfname, 1024);
   }
 
   /*
@@ -1066,7 +1111,7 @@ UUInfoFile (uulist *thefile, void *opaque,
 
   while (!feof (inpfile) && 
 	 (uu_fast_scanning || ftell(inpfile) < maxpos)) {
-    if (FP_fgets (uugen_inbuffer, 511, inpfile) == NULL)
+    if (_FP_fgets (uugen_inbuffer, 511, inpfile) == NULL)
       break;
     uugen_inbuffer[511] = '\0';
 
@@ -1117,14 +1162,14 @@ UURenameFile (uulist *thefile, char *newname)
 
   oldname = thefile->filename;
 
-  if ((thefile->filename = FP_strdup (newname)) == NULL) {
+  if ((thefile->filename = _FP_strdup (newname)) == NULL) {
     UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_NOT_RENAME),
 	       oldname, newname);
     thefile->filename = oldname;
     return UURET_NOMEM;
   }
-  FP_free (oldname);
+  _FP_free (oldname);
   return UURET_OK;
 }
 
@@ -1141,7 +1186,7 @@ UURemoveTemp (uulist *thefile)
 		 thefile->binfile,
 		 strerror (uu_errno = errno));
     }
-    FP_free (thefile->binfile);
+    _FP_free (thefile->binfile);
     thefile->binfile = NULL;
     thefile->state  &= ~UUFILE_TMPFILE;
   }
@@ -1152,30 +1197,58 @@ int UUEXPORT
 UUCleanUp (void)
 {
   itbd *iter=ftodel, *ptr;
+  uulist *liter;
+  uufile *fiter;
   allomap *aiter;
 
-  UUkilllist (UUGlobalFileList);
-  UUGlobalFileList = NULL;
-
   /*
-   * delete input files
+   * delete temporary input files (such as the copy of stdin)
    */
+
   while (iter) {
     if (unlink (iter->fname)) {
       UUMessage (uulib_id, __LINE__, UUMSG_WARNING,
 		 uustring (S_TMP_NOT_REMOVED),
 		 iter->fname, strerror (uu_errno = errno));
     }
-    FP_free (iter->fname);
+    _FP_free (iter->fname);
     ptr  = iter;
     iter = iter->NEXT;
-    FP_free (ptr);
+    _FP_free (ptr);
   }
+
   ftodel = NULL;
 
-  FP_free (uusavepath);
-  FP_free (uuencodeext);
-  FP_free (sstate.source);
+  /*
+   * Delete input files after successful decoding
+   */
+
+  if (uu_remove_input) {
+    liter = UUGlobalFileList;
+    while (liter) {
+      if (liter->state & UUFILE_DECODED) {
+	fiter = liter->thisfile;
+	while (fiter) {
+	  if (fiter->data && fiter->data->sfname) {
+	    /*
+	     * Error code ignored. We might want to delete a file multiple
+	     * times
+	     */
+	    unlink (fiter->data->sfname);
+	  }
+	  fiter = fiter->NEXT;
+	}
+      }
+      liter = liter->NEXT;
+    }
+  }
+
+  UUkilllist (UUGlobalFileList);
+  UUGlobalFileList = NULL;
+
+  _FP_free (uusavepath);
+  _FP_free (uuencodeext);
+  _FP_free (sstate.source);
 
   uusavepath  = NULL;
   uuencodeext = NULL;
@@ -1188,7 +1261,7 @@ UUCleanUp (void)
   while (mssdepth) {
     mssdepth--;
     UUkillheaders (&(multistack[mssdepth].envelope));
-    FP_free (multistack[mssdepth].source);
+    _FP_free (multistack[mssdepth].source);
   }
 
   /*
@@ -1196,7 +1269,7 @@ UUCleanUp (void)
    */
 
   for (aiter=toallocate; aiter->ptr; aiter++) {
-    FP_free (*(aiter->ptr));
+    _FP_free (*(aiter->ptr));
     *(aiter->ptr) = NULL;
   }
 
